/* Negotiate client side authentication. */

#include "tcpbroker.h"

char *Progname;
char Usage[] = "usage: %s host port\n";
#define USAGE() fprintf(stderr, Usage, Progname)

int Fd[4];
int Maxfd;

void doit(char *, char *);
int wait_for_input();
void cleanup(int sock);

int main(int argc, char **argv)
{
	int c;

	Progname = *argv;

	/* Parse option arguments. */

	while ((c = getopt(argc, argv, "")) != EOF) {
		switch (c) {

		default:
			USAGE();
			exit(1);
		}
	}
	argc -= optind;
	argv += optind;

	if (argc != 2) {
		USAGE();
		exit(1);
	}

	doit(argv[0], argv[1]);

	return 0;
}

void doit(char *host, char *port)
{
	int i, j, sock, n, clen;
	char buf[1024];
	struct sockaddr_in sock_name;
	struct hostent *hostent;

	/* Get the host's address. */

	if ((hostent = gethostbyname(host)) == NULL) {
		fprintf(stderr, "error resolving hostname %s\n", host);
		exit(1);
	}

	/* Open a socket. */

	if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		perror("socket");
		exit(1);
	}

	/* Assign the socket's address and port. */

	memset(&sock_name, 0, sizeof(sock_name));
	sock_name.sin_family = AF_INET;
	sock_name.sin_port = htons(atoi(port));
	sock_name.sin_addr = *(struct in_addr *)hostent->h_addr_list[0];

	/* Try to connect. */

	if (connect(sock, (struct sockaddr *)&sock_name, sizeof(sock_name))) {
		perror("connect");
		cleanup(sock);
	}

	/* Set up the fds for copythru -- we'll read from 0 & 1 and write
	to 2 & 3, resp. */

	Fd[0] = sock;
	Fd[1] = fileno(stdin);
	Fd[2] = fileno(stdout);
	Fd[3] = sock;
	Maxfd = Fd[0] > Fd[1] ? Fd[0] : Fd[1];
	Maxfd++;

#if 0
	/* This is where we would issue commands to a proxy, e.g., */

	strcpy(buf, "connect hostname 23\n");
	write(sock, buf, strlen(buf));
	/* it may be necessary to sleep a bit before sending the cookie if
	the proxy likes to flush the socket after getting connected */
	sleep(15);
#endif

	/* Indicate that we are willing to authenticate. */

	write(sock, TSRP_COOKIE, TSRP_COOKIE_LEN);

	/* Wait for the reply cookie. */

	clen = 0;
	while (1) {
		i = wait_for_input();
		if (i >= 0) {
			n = read(Fd[i], buf, sizeof(buf));
			if (n == 0) {
				break;
			}
			if (i == 1) {
				write(sock, buf, n);
				continue;
			}

			/* Look for the cookie.  It'll be last. */

			for (j = 0; j < n; j++) {
				if (buf[j] == TSRP_COOKIE[clen]) {
					clen++;
//                                        if (clen == TSRP_COOKIE_LEN) {
//                                                break;
//                                        }
				} else {
					if (clen) {
						write(Fd[2], TSRP_COOKIE, clen);
					}
					write(Fd[2], &buf[j], 1);
					clen = 0;
				}
			}
			if (clen == TSRP_COOKIE_LEN) {
				break;
			}
		}
	}

	/* We're synched now.  Begin authentication. */

	if (!tsrp_client_authenticate(sock, "broker", NULL, NULL)) {
		fprintf(stderr, "authentication failed\n");
		cleanup(sock);
	}
	buf[0] = '\r';          /* output a newline after the password prompt */
	buf[1] = '\n';
	write(Fd[2], buf, 2);

	while (1) {
		i = wait_for_input();
		if (i >= 0) {
			n = read(Fd[i], buf, sizeof(buf));
			if (n == 0 || (i == 1 && buf[0] == 0x04)) {
				break;
			}
			write(Fd[i + 2], buf, n);
		}
	}
}

int wait_for_input()
{
	int i;
	fd_set fds;
	struct timeval tv;
	static int save1 = 0;

	if (save1) {
		save1 = 0;
		return 1;
	}

	FD_ZERO(&fds);
	FD_SET(Fd[0], &fds);
	FD_SET(Fd[1], &fds);
	tv.tv_sec = 60;
	tv.tv_usec = 0;
	if (select(Maxfd, &fds, NULL, NULL, &tv) < 0) {
		return -1;
	}

	i = -1;
	if (FD_ISSET(Fd[0], &fds)) {
		i = 0;
	}
	if (FD_ISSET(Fd[1], &fds)) {
		if (i < 0) {
			i = 1;
		} else {
			save1 = 1;
		}
	}

	return i;
}

void cleanup(int sock)
{
	close(sock);
	exit(1);
}
